/****************************************************************************
          Copyright (C) 1995, University of California, Davis

          THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND THE UNIVERSITY
          OF CALIFORNIA DOES NOT MAKE ANY WARRANTY ABOUT THE SOFTWARE, ITS
          PERFORMANCE, ITS MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
          USE, FREEDOM FROM ANY COMPUTER DISEASES OR ITS CONFORMITY TO ANY
          SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF
          THE SOFTWARE IS WITH THE USER.

          Copyright of the software and supporting documentation is
          owned by the University of California, and free access
          is hereby granted as a license to use this software, copy this
          software and prepare derivative works based upon this software.
          However, any distribution of this software source code or
          supporting documentation or derivative works (source code and
          supporting documentation) must include this copyright notice.
****************************************************************************/

/***************************************************************************
 *
 * University of California, Davis
 * UCDMC DICOM Network Transport Libraries
 * Version 0.1 Beta
 *
 * Technical Contact: mhoskin@ucdavis.edu
 *
 ***************************************************************************/


class	DICOMObject;

//##ModelId=38F5FEFD0012
class	VR
	{
	public:
	//##ModelId=38F5FEFD0149
		UINT16	Group;
	//##ModelId=38F5FEFD0135
		UINT16	Element;
	//##ModelId=38F5FEFD0121
		UINT32	Length;
	//##ModelId=38F5FEFD00F9
		void		*Data;
	//##ModelId=38F5FEFD00E5
		BOOL		ReleaseMemory;
	//##ModelId=38F5FEFD00D1
		BOOL		Valid;
		// void pointers for historical reasons.  SQObjectArray
		// is problem best moved to an Array < DICOMObject * > * in
		// the future once the various system compilers work around
		// the linking problems...  ReferentialIntegrityDatabase should
		// remain void because it is not defined unless the (optional)
		// IOD library is linked in.
	//##ModelId=38F5FEFD00BD
		void		*SQObjectArray;
	//##ModelId=38F5FEFD00A8
		void		*ReferentialIntegrityDatabase;
	//##ModelId=38F5FEFD00A0
		UINT16		TypeCode;
	//##ModelId=38F5FEFD0257
		VR() { Data = NULL; Length = 0; ReleaseMemory = TRUE; TypeCode = 0;
				SQObjectArray = NULL; Valid = FALSE; };
	//##ModelId=38F5FEFD0261
		VR(UINT16 g, UINT16 e, UINT32 l, BOOL	Alloc);
	//##ModelId=38F5FEFD02ED
		VR(UINT16 g, UINT16 e, UINT32 l, void	*d);
	//##ModelId=38F5FEFD0384
		VR(UINT16 g, UINT16 e, UINT32 l, void	*d, BOOL);
	//##ModelId=38F5FEFD0243
		~VR();
	//##ModelId=38F5FEFD021B
		UINT		operator	>	(VR	&vr);
	//##ModelId=38F5FEFD01E9
		UINT		operator	<	(VR	&vr);
	//##ModelId=38F5FEFD01B7
		UINT		operator	==	(VR	&vr);

		// To support SetIf/Morph

	//##ModelId=38F5FEFD018F
		BOOL		SetIf(VR	*);
	//##ModelId=38F5FEFD0167
		BOOL		Morph(DICOMObject	*);
	//##ModelId=38F5FEFD0153
		BOOL		Reset ();
	};

//##ModelId=38F5FEE301D8
class	VRGroupPQueue	:
	public	PQueueOfPtr<VR	*>
	{
	public:
	//##ModelId=38F5FEE302D4
		UINT16	Group;
	//##ModelId=38F5FEE302C8
		UINT32	Length;
	//##ModelId=38F5FEE302B4
		VR			*Element0;

	public:
	//##ModelId=38F5FEE3034A
		BOOL	Push(VR	*vr);
	//##ModelId=38F5FEE30340
		VR		*Pop();
	//##ModelId=38F5FEE30336
		VR		*GroupLength();
	//##ModelId=38F5FEE30322
		UINT		operator	>	(VRGroupPQueue	&vrgroup);
	//##ModelId=38F5FEE30304
		UINT		operator	<	(VRGroupPQueue	&vrgroup);
	//##ModelId=38F5FEE302F0
		UINT		operator	==	(VRGroupPQueue	&vrgroup);
	//##ModelId=38F5FEE302E6
		VRGroupPQueue(UINT16	G)	{ Group = G; Length = 0; };
	};

//##ModelId=38F5FF1302DB
class	DICOMObject	:
	public	PQueueOfPtr<VRGroupPQueue *>
	{
	VRGroupPQueue	*CurrentGroup, *LastVRG;
	//##ModelId=38F5FF15002B
	VR				*LastVR;

	//##ModelId=38F5FF15000D
		UINT16					LastGroup;
	//##ModelId=38F5FF1403D7
		UINT16					LastElement;

	public:
	//##ModelId=38F5FF1403C3
		UINT32					Length;
	//##ModelId=38F5FF140391
		BOOL					Packaged;
	//##ModelId=38F5FF140373
		Array<VRGroupPQueue *>	VRGroupPQueues;
	//##ModelId=38F5FF140246
		Array<DICOMObject *>	*SQObjectArray;
	//##ModelId=38F5FF140142
		UINT32					FreeSQObjectSize;
	//##ModelId=38F5FF140110
		DICOMObject				*AutoMakeDO;
	//##ModelId=38F5FF1400FE
		UINT32					FixedSize;
	//##ModelId=38F5FF1400E8
		BOOL					UseFixedSize;
	private:
	//##ModelId=38F5FF1501BB
		BOOL		Package();
	public:
	//##ModelId=38F5FF15012F
		virtual		BOOL		Push(VR	*vr);
	//##ModelId=38F5FF150175
		virtual		BOOL		Push(DICOMObject *);	// Sequence Embedding
	//##ModelId=38F5FF15011B
		virtual		VR			*Pop();
	//##ModelId=38F5FF1500B7
		LE_UINT16	GetUINT16(UINT16, UINT16);
	//##ModelId=38F5FF15005D
		VR			*GetVR(UINT16, UINT16);
	//##ModelId=38F5FF150049
		virtual		BOOL		Reset();
	//##ModelId=38F5FF15003F
		DICOMObject();
	//##ModelId=38F5FF150035
		~DICOMObject();
	};

//##ModelId=38F5FED50155
class	DICOMCommandObject	:
	public	DICOMObject
	{
	};

//##ModelId=38F5FEBA03D7
class	DICOMDataObject	:
	public	DICOMObject
	{
	};
