/****************************************************************************
          Copyright (C) 1995, University of California, Davis

          THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND THE UNIVERSITY
          OF CALIFORNIA DOES NOT MAKE ANY WARRANTY ABOUT THE SOFTWARE, ITS
          PERFORMANCE, ITS MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
          USE, FREEDOM FROM ANY COMPUTER DISEASES OR ITS CONFORMITY TO ANY
          SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF
          THE SOFTWARE IS WITH THE USER.

          Copyright of the software and supporting documentation is
          owned by the University of California, and free access
          is hereby granted as a license to use this software, copy this
          software and prepare derivative works based upon this software.
          However, any distribution of this software source code or
          supporting documentation or derivative works (source code and
          supporting documentation) must include this copyright notice.
****************************************************************************/

/***************************************************************************
 *
 * University of California, Davis
 * UCDMC DICOM Network Transport Libraries
 * Version 0.1 Beta
 *
 * Technical Contact: mhoskin@ucdavis.edu
 *
 ***************************************************************************/

/***********************************************************************
 *
 *
 ***********************************************************************/
#define	DEFAULT_ACCEPTANCE	TRUE


// These are Read/Write file types
#	define	ACRNEMA_VR_DUMP				1
#	define	DICOM_CHAPTER_10_IMPLICIT	2
#	define	DICOM_CHAPTER_10_EXPLICIT	3

// These are "Transfer Syntax Alias" types.
enum
	{
	TSA_IMPLICIT_LITTLE_ENDIAN = 100,
	TSA_EXPLICIT_LITTLE_ENDIAN,
	TSA_EXPLICIT_BIG_ENDIAN
	};

//##ModelId=38F5FEF801F6
class	FileBuffer	:
	public	LinkedBuffer
	{
	public:
	//##ModelId=38F5FEF8026E
		FILE	*fp;
	//##ModelId=38F5FEF802DC
		BOOL	OpenForRead ( char	*filename );
	//##ModelId=38F5FEF802C8
		BOOL	OpenForWrite ( char *filename );
	//##ModelId=38F5FEF802BE
		BOOL	Close ();
	//##ModelId=38F5FEF80296
		INT		ReadBinary(BYTE *Data, UINT Count);
	//##ModelId=38F5FEF80278
		BOOL	SendBinary(BYTE *Data, UINT Count);
	};

//##ModelId=38F5FECA0181
class	PDU_Service	:
	public	Buffer,			// Transport Buffer
	public	AAssociateRQ,	// Associate Request
	public	AAssociateAC,	// Associate Accept
	public	AAssociateRJ,	// Associate Reject
	public	AReleaseRQ,		// Associate Release
	public	AReleaseRP,		// Associate Release Response
	public	AAbortRQ,		// Associate Abort Request
	public	PDataTF			// P-DATA Service
	{
	protected:
	//##ModelId=38F5FECA0312
		Array<AbstractSyntax>				ProposedAbstractSyntaxs;
	//##ModelId=38F5FECA027C
		Array<PresentationContextAccept>	AcceptedPresentationContexts;
	//##ModelId=38F5FECA0260
		UINT32								SizeCap;
	//##ModelId=38F5FECA024A
		RTC									*AttachedRTC;
	//##ModelId=38F5FECA022C
		BOOL								DestructAttachedRTC;
	protected:
	//##ModelId=38F5FED500AB
		BOOL			InterogateAAssociateRQ();
	//##ModelId=38F5FED50029
		BOOL			InterogateAAssociateAC();
	//##ModelId=38F5FED402E4
		virtual	BOOL	CanYouHandleTransferSyntax(TransferSyntax &);
	//##ModelId=38F5FED401D6
		virtual	BOOL	ShouldIAcceptRemoteApTitle(BYTE	*)
			{ return ( DEFAULT_ACCEPTANCE ); };
	//##ModelId=38F5FED400C8
		virtual	BOOL	ShouldIAcceptLocalApTitle(BYTE *)
			{ return ( DEFAULT_ACCEPTANCE ); };
	//##ModelId=38F5FED303A1
		virtual	BOOL	ShouldIAcceptApplicationContext(ApplicationContext &)
			{ return ( DEFAULT_ACCEPTANCE ); };
	//##ModelId=38F5FED30257
		virtual	BOOL	ShouldIAcceptPresentationContext(
							PresentationContext &,
							PresentationContextAccept &)
			{ return ( DEFAULT_ACCEPTANCE ); };
	//##ModelId=38F5FED30166
		virtual	BOOL	ShouldIAcceptAbstractSyntax(AbstractSyntax &)
			{ return ( DEFAULT_ACCEPTANCE ); };
	//##ModelId=38F5FED30044
		virtual	BOOL	AddTransferSyntaxs(PresentationContext &);
	//##ModelId=38F5FED20332
		virtual	BOOL	GetImplementationClass(ImplementationClass &);
	//##ModelId=38F5FED20237
		virtual	BOOL	GetImplementationVersion(ImplementationVersion &);

		// Entry Point Functions
	//##ModelId=38F5FED20075
		virtual	BOOL	ParseRawVRIntoDCM(BYTE, LinkedBuffer &, DICOMObject *);
	//##ModelId=38F5FED1016E
		virtual	BYTE	ParseDCMIntoRawVR(DICOMObject *, LinkedBuffer &);
	//##ModelId=38F5FED102B8
		virtual	BYTE	ParseDCMIntoRawVR(DICOMObject *, LinkedBuffer &, UID &);

		// Implicit Encoders
	//##ModelId=38F5FED10023
		virtual	BOOL	Implicit_ParseRawVRIntoDCM(
							LinkedBuffer &, DICOMObject *);
	//##ModelId=38F5FED002DF
		virtual	BOOL	Implicit_ParseDCMIntoRawVR
							(DICOMObject *, LinkedBuffer &);
	//##ModelId=38F5FED001A8
		virtual	BOOL	ImplicitLittleEndian_ParseRawVRIntoDCM(
							LinkedBuffer &, DICOMObject *);
	//##ModelId=38F5FED0007C
		virtual	BOOL	ImplicitLittleEndian_ParseDCMIntoRawVR
							(DICOMObject *, LinkedBuffer &);

		// Explicit Decoders
	//##ModelId=38F5FECF0341
		virtual	BOOL	Explicit_ParseRawVRIntoDCM(
							LinkedBuffer &, DICOMObject *);
	//##ModelId=38F5FECF0233
		virtual	BOOL	Explicit_ParseDCMIntoRawVR
							(DICOMObject *, LinkedBuffer &);
	//##ModelId=38F5FECF0110
		virtual	BOOL	ExplicitLittleEndian_ParseRawVRIntoDCM(
							LinkedBuffer &, DICOMObject *);
	//##ModelId=38F5FECE03E0
		virtual	BOOL	ExplicitLittleEndian_ParseDCMIntoRawVR
							(DICOMObject *, LinkedBuffer &);
	//##ModelId=38F5FECE02F0
		virtual	BOOL	ExplicitBigEndian_ParseRawVRIntoDCM(
							LinkedBuffer &, DICOMObject *);
	//##ModelId=38F5FECE0209
		virtual	BOOL	ExplicitBigEndian_ParseDCMIntoRawVR
							(DICOMObject *, LinkedBuffer &);

		// Used to Read DICOM Files (all types)
	//##ModelId=38F5FECE00E7
		virtual	BOOL	Dynamic_ParseRawVRIntoDCM
							(LinkedBuffer &, DICOMObject *, UINT StartMode);

		// Used to write Chapter 10 Explcit (new) files
	//##ModelId=38F5FECD03AD
		virtual	BOOL	Dynamic_ParseDCMIntoRawVR
							(DICOMObject *, LinkedBuffer &, UINT StartMode);

		// Image Pixel Data encapsulation routines

	//##ModelId=38F5FECD02E4
		virtual	BOOL	ParseImagePixelDataDCMToRaw
							(Array<DICOMDataObject*>	*, LinkedBuffer &);
	//##ModelId=38F5FECD021C
		virtual	BOOL	ParseImagePixelDataRawToDCM
							(LinkedBuffer	&, DICOMObject	*);

		// Support Routines for Transfer Syntax Encoding/Decoding
	public:	// The following routine is usefull to check for a valid
			// Accepted Presentation Context UID
	//##ModelId=38F5FECD01C2
				BYTE	GetAcceptedPCIDForTransferSyntax(UID &TrnUID);
	//##ModelId=38F5FECD0136
				BOOL	GetTransferSyntaxUID(BYTE, UID &TrnUID);
	//##ModelId=38F5FECD0045
				BYTE	GetPresentationContextID(UID &AbsUID);
	//##ModelId=38F5FECD00A9
				BYTE	GetPresentationContextID(UID &AbsUID, UID &TrnUID);
	public:
	//##ModelId=38F5FECA021A
		Socket	Link;

		// RTC Support (optional, helps transfer syntax engines)
	//##ModelId=38F5FECC03AB
				BOOL	AttachRTC(RTC *, BOOL	Destruct = FALSE);
	//##ModelId=38F5FECC038D
				RTC		*DetachRTC();
		// Wrap data over from the Associate/PData Classes over to the buffer
	//##ModelId=38F5FECC02BB
				BOOL	Read(BYTE	*Data, UINT	Count)
			{	return ( Buffer :: Read ( Data, Count ) ); };
	//##ModelId=38F5FECC004E
				BOOL	Write(BYTE	*Data, UINT	Count)
			{	return ( Buffer :: Write ( Data, Count ) ); };
	//##ModelId=38F5FECC0008
				BOOL	Kill(UINT	Count)
			{	return ( Buffer :: Kill ( Count ) ); };
	//##ModelId=38F5FECB03D2
				BOOL	Flush()
			{	return ( Buffer :: Flush () ); };
		// Wrap data over from the buffer over to the socket
	//##ModelId=38F5FECB0364
				INT	ReadBinary(BYTE *Data, UINT Count)
			{	return ( Link.ReadBinary(Data, Count) ); };
	//##ModelId=38F5FECB02F5
				BOOL	SendBinary(BYTE *Data, UINT Count)
			{	return ( Link.SendBinary(Data, Count) ); };

	//##ModelId=38F5FECB02A5
		virtual	BOOL	MakeChapter10(DICOMDataObject *);
	//##ModelId=38F5FECB025F
		DICOMDataObject	*LoadDICOMDataObject ( char	* );
	//##ModelId=38F5FECB01DD
		BOOL	SaveDICOMDataObject ( char	*, UINT, DICOMDataObject	* );
	//##ModelId=38F5FECB01BF
		BOOL	ClearAbstractSyntaxs();
	//##ModelId=38F5FECB0179
		virtual	BOOL	AddAbstractSyntax(UID &);
	//##ModelId=38F5FECB0115
		virtual	BOOL	AddAbstractSyntaxAlias(UID &Source, UID &Alias);
	//##ModelId=38F5FECB00D9
		virtual	BOOL	SetApplicationContext(UID &);
	//##ModelId=38F5FECB00A7
		virtual	BOOL	SetLocalAddress(BYTE *);
	//##ModelId=38F5FECB006B
		virtual	BOOL	SetRemoteAddress(BYTE *);
	//##ModelId=38F5FECB001A
		BOOL	Connect(BYTE *, BYTE *);
		
	//##ModelId=38F5FECB0010
		BOOL	Close();

	//##ModelId=38F5FECA03B2
		BOOL	Listen(BYTE *);

	//##ModelId=38F5FECC033D
		BOOL	Read(DICOMObject	*);
	//##ModelId=38F5FECC010C
		BOOL	Write(DICOMCommandObject	*);
	//##ModelId=38F5FECC015C
		BOOL	Write(DICOMDataObject	*);
	//##ModelId=38F5FECC01A2
		BOOL	Write(DICOMCommandObject	*, UID	&);
	//##ModelId=38F5FECC0225
		BOOL	Write(DICOMDataObject	*, UID	&);
	//##ModelId=38F5FECA036C
		BOOL	Multiplex(int);

	//##ModelId=38F5FECA038A
		int		Multiplex(BYTE *);
	//##ModelId=38F5FECA03D0
		int		Listen(int);
	//##ModelId=38F5FECA034E
		BOOL	SetTimeOut(int TimeOut)
			{ return ( Link.SetTimeOut(TimeOut) ); };
	//##ModelId=38F5FECA0344
				PDU_Service();
	//##ModelId=38F5FECA0330
		virtual	~PDU_Service();
	};


