/****************************************************************************
          Copyright (C) 1995, University of California, Davis

          THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND THE UNIVERSITY
          OF CALIFORNIA DOES NOT MAKE ANY WARRANTY ABOUT THE SOFTWARE, ITS
          PERFORMANCE, ITS MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
          USE, FREEDOM FROM ANY COMPUTER DISEASES OR ITS CONFORMITY TO ANY
          SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF
          THE SOFTWARE IS WITH THE USER.

          Copyright of the software and supporting documentation is
          owned by the University of California, and free access
          is hereby granted as a license to use this software, copy this
          software and prepare derivative works based upon this software.
          However, any distribution of this software source code or
          supporting documentation or derivative works (source code and
          supporting documentation) must include this copyright notice.
****************************************************************************/

/***************************************************************************
 *
 * University of California, Davis
 * UCDMC DICOM Network Transport Libraries
 * Version 0.1 Beta
 *
 * Technical Contact: mhoskin@ucdavis.edu
 *
 ***************************************************************************/

// Storage classes

// Note there are many "ungood" things about the class hierachys in
// this file.  Notable things should be protected rather than public,
// there should be no inline "wrapper"'s..  Both of these are done
// to be compatible with gcc.

// Base Storage class

//##ModelId=38F5FED703D9
class	StandardStorage	:
	public	CStoreRQ,
	public	CStoreRSP
	{
	public:
	//##ModelId=38F5FED80131
		BOOL	GetUID ( UID &uid) { return (uGetUID(uid)); };
	//##ModelId=38F5FED800AF
		virtual	BOOL	uGetUID ( UID &uid ) = 0;
	//##ModelId=38F5FED8009B
		virtual	UINT16	CheckObject(DICOMDataObject * ) { return ( 0 ); };
	//##ModelId=38F5FED80069
		BOOL	Read (	PDU_Service *,
						DICOMCommandObject *, 
						DICOMDataObject * );
	//##ModelId=38F5FED8004B
		BOOL	Write (	PDU_Service	*, DICOMDataObject	* );
	};

//##ModelId=38F5FF0303E6
class	UnknownStorage	:
	public	CStoreRQ,
	public	CStoreRSP
	{
	public:
	//##ModelId=38F5FF0400D2
		BOOL	GetUID(UID &);
	//##ModelId=38F5FF0400BE
		virtual	UINT16	CheckObject(DICOMDataObject * ) { return ( 0 ); };

	//##ModelId=38F5FF0400A9
		BOOL	Read (	PDU_Service	*,
					DICOMCommandObject	*,
					DICOMDataObject	* );
	};

//##ModelId=38F5FEF100E7
class	CRStorage	:
	public	StandardStorage
	{
	public:
	//##ModelId=38F5FEF1019C
		BOOL	GetUID(UID &);
	//##ModelId=38F5FEF10192
		BOOL	uGetUID(UID &uid) { return ( GetUID(uid) ); };
	//##ModelId=38F5FEF1016A
		inline	Read ( PDU_Service *PDU, DICOMCommandObject *DCO,
					DICOMDataObject *DDO )
			{ return ( StandardStorage :: Read ( PDU, DCO, DDO ) ); };
	//##ModelId=38F5FEF1014B
		inline	Write ( PDU_Service *PDU, DICOMDataObject *DDO)
			{ return ( StandardStorage :: Write ( PDU, DDO ) ); };
	};

//##ModelId=38F5FEE4000C
class	CTStorage	:
	public	StandardStorage
	{
	public:
	//##ModelId=38F5FEE400AD
		BOOL	GetUID(UID &);
	//##ModelId=38F5FEE40099
		BOOL	uGetUID(UID &uid) { return ( GetUID(uid) ); };
	//##ModelId=38F5FEE4007A
		inline	Read ( PDU_Service *PDU, DICOMCommandObject *DCO,
					DICOMDataObject *DDO )
			{ return ( StandardStorage :: Read ( PDU, DCO, DDO ) ); };
	//##ModelId=38F5FEE40066
		inline	Write ( PDU_Service *PDU, DICOMDataObject *DDO)
			{ return ( StandardStorage :: Write ( PDU, DDO ) ); };
	};

//##ModelId=38F5FED801DC
class	USMultiframeStorage	:
	public	StandardStorage
	{
	public:
	//##ModelId=38F5FED8027C
		BOOL	GetUID(UID &);
	//##ModelId=38F5FED80272
		BOOL	uGetUID(UID &uid) { return ( GetUID(uid) ); };
	//##ModelId=38F5FED8024A
		inline	Read ( PDU_Service *PDU, DICOMCommandObject *DCO,
					DICOMDataObject *DDO )
			{ return ( StandardStorage :: Read ( PDU, DCO, DDO ) ); };
	//##ModelId=38F5FED8022D
		inline	Write ( PDU_Service *PDU, DICOMDataObject *DDO)
			{ return ( StandardStorage :: Write ( PDU, DDO ) ); };
	};

//##ModelId=38F5FEF10232
class	MRStorage	:
	public	StandardStorage
	{
	public:
	//##ModelId=38F5FEF102DC
		BOOL	GetUID(UID &);
	//##ModelId=38F5FEF102D2
		BOOL	uGetUID(UID &uid) { return ( GetUID(uid) ); };
	//##ModelId=38F5FEF102B4
		inline	Read ( PDU_Service *PDU, DICOMCommandObject *DCO,
					DICOMDataObject *DDO )
			{ return ( StandardStorage :: Read ( PDU, DCO, DDO ) ); };
	//##ModelId=38F5FEF102A0
		inline	Write ( PDU_Service *PDU, DICOMDataObject *DDO)
			{ return ( StandardStorage :: Write ( PDU, DDO ) ); };
	};

//##ModelId=38F5FEF1037C
class	NMStorage	:
	public	StandardStorage
	{
	public:
	//##ModelId=38F5FEF2003F
		BOOL	GetUID(UID &);
	//##ModelId=38F5FEF2002A
		BOOL	uGetUID(UID &uid) { return ( GetUID(uid) ); };
	//##ModelId=38F5FEF2000C
		inline	Read ( PDU_Service *PDU, DICOMCommandObject *DCO,
					DICOMDataObject *DDO )
			{ return ( StandardStorage :: Read ( PDU, DCO, DDO ) ); };
	//##ModelId=38F5FEF103E0
		inline	Write ( PDU_Service *PDU, DICOMDataObject *DDO)
			{ return ( StandardStorage :: Write ( PDU, DDO ) ); };
	};

//##ModelId=38F5FEE40143
class	USStorage	:
	public	StandardStorage
	{
	public:
	//##ModelId=38F5FEE401ED
		BOOL	GetUID(UID &);
	//##ModelId=38F5FEE401D9
		BOOL	uGetUID(UID &uid) { return ( GetUID(uid) ); };
	//##ModelId=38F5FEE401BB
		inline	Read ( PDU_Service *PDU, DICOMCommandObject *DCO,
					DICOMDataObject *DDO )
			{ return ( StandardStorage :: Read ( PDU, DCO, DDO ) ); };
	//##ModelId=38F5FEE401A7
		inline	Write ( PDU_Service *PDU, DICOMDataObject *DDO)
			{ return ( StandardStorage :: Write ( PDU, DDO ) ); };
	};

//##ModelId=38F5FEBE02D9
class	SCStorage	:
	public	StandardStorage
	{
	public:
	//##ModelId=38F5FEBE0351
		BOOL	GetUID(UID &);
	//##ModelId=38F5FEBE033E
		BOOL	uGetUID(UID &uid) { return ( GetUID(uid) ); };
	//##ModelId=38F5FEBE0320
		inline	Read ( PDU_Service *PDU, DICOMCommandObject *DCO,
					DICOMDataObject *DDO )
			{ return ( StandardStorage :: Read ( PDU, DCO, DDO ) ); };
	//##ModelId=38F5FEBE030C
		inline	Write ( PDU_Service *PDU, DICOMDataObject *DDO)
			{ return ( StandardStorage :: Write ( PDU, DDO ) ); };
	};

//##ModelId=38F5FEED0132
class	StandaloneOverlayStorage	:
	public	StandardStorage
	{
	public:
	//##ModelId=38F5FEED01F0
		BOOL	GetUID(UID &);
	//##ModelId=38F5FEED01E6
		BOOL	uGetUID(UID &uid) { return ( GetUID(uid) ); };
	//##ModelId=38F5FEED01C8
		inline	Read ( PDU_Service *PDU, DICOMCommandObject *DCO,
					DICOMDataObject *DDO )
			{ return ( StandardStorage :: Read ( PDU, DCO, DDO ) ); };
	//##ModelId=38F5FEED01AA
		inline	Write ( PDU_Service *PDU, DICOMDataObject *DDO)
			{ return ( StandardStorage :: Write ( PDU, DDO ) ); };
	};

//##ModelId=38F5FEF9020B
class	StandaloneCurveStorage	:
	public	StandardStorage
	{
	public:
	//##ModelId=38F5FEF902D4
		BOOL	GetUID(UID &);
	//##ModelId=38F5FEF902B5
		BOOL	uGetUID(UID &uid) { return ( GetUID(uid) ); };
	//##ModelId=38F5FEF90297
		inline	Read ( PDU_Service *PDU, DICOMCommandObject *DCO,
					DICOMDataObject *DDO )
			{ return ( StandardStorage :: Read ( PDU, DCO, DDO ) ); };
	//##ModelId=38F5FEF90283
		inline	Write ( PDU_Service *PDU, DICOMDataObject *DDO)
			{ return ( StandardStorage :: Write ( PDU, DDO ) ); };
	};

//##ModelId=38F5FEE7024B
class	StandaloneModalityLUTStorage	:
	public	StandardStorage
	{
	public:
	//##ModelId=38F5FEE702F6
		BOOL	GetUID(UID &);
	//##ModelId=38F5FEE702E2
		BOOL	uGetUID(UID &uid) { return ( GetUID(uid) ); };
	//##ModelId=38F5FEE702C4
		inline	Read ( PDU_Service *PDU, DICOMCommandObject *DCO,
					DICOMDataObject *DDO )
			{ return ( StandardStorage :: Read ( PDU, DCO, DDO ) ); };
	//##ModelId=38F5FEE702B0
		inline	Write ( PDU_Service *PDU, DICOMDataObject *DDO)
			{ return ( StandardStorage :: Write ( PDU, DDO ) ); };
	};

//##ModelId=38F5FF1600AE
class	StandaloneVOILUTStorage	:
	public	StandardStorage
	{
	public:
	//##ModelId=38F5FF1601DB
		BOOL	GetUID(UID &);
	//##ModelId=38F5FF1601C7
		BOOL	uGetUID(UID &uid) { return ( GetUID(uid) ); };
	//##ModelId=38F5FF1601A9
		inline	Read ( PDU_Service *PDU, DICOMCommandObject *DCO,
					DICOMDataObject *DDO )
			{ return ( StandardStorage :: Read ( PDU, DCO, DDO ) ); };
	//##ModelId=38F5FF160195
		inline	Write ( PDU_Service *PDU, DICOMDataObject *DDO)
			{ return ( StandardStorage :: Write ( PDU, DDO ) ); };
	};


//##ModelId=38F5FF03024C
class	GEMRStorage	:
	public	StandardStorage
	{
	public:
	//##ModelId=38F5FF030332
		BOOL	GetUID(UID &);
	//##ModelId=38F5FF030328
		BOOL	uGetUID(UID &uid) { return ( GetUID(uid) ); };
	//##ModelId=38F5FF03030A
		inline	Read ( PDU_Service *PDU, DICOMCommandObject *DCO,
					DICOMDataObject *DDO )
			{ return ( StandardStorage :: Read ( PDU, DCO, DDO ) ); };
	//##ModelId=38F5FF0302EC
		inline	Write ( PDU_Service *PDU, DICOMDataObject *DDO)
			{ return ( StandardStorage :: Write ( PDU, DDO ) ); };
	};


//##ModelId=38F5FEEC03CF
class	GECTStorage	:
	public	StandardStorage
	{
	public:
	//##ModelId=38F5FEED0091
		BOOL	GetUID(UID &);
	//##ModelId=38F5FEED007E
		BOOL	uGetUID(UID &uid) { return ( GetUID(uid) ); };
	//##ModelId=38F5FEED0060
		inline	Read ( PDU_Service *PDU, DICOMCommandObject *DCO,
					DICOMDataObject *DDO )
			{ return ( StandardStorage :: Read ( PDU, DCO, DDO ) ); };
	//##ModelId=38F5FEED004B
		inline	Write ( PDU_Service *PDU, DICOMDataObject *DDO)
			{ return ( StandardStorage :: Write ( PDU, DDO ) ); };
	};


